# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/agent/message/proto/v3/control/control.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5ostorlab/agent/message/proto/v3/control/control.proto\x12\'ostorlab.agent.message.proto.v3.control\"\x19\n\x07\x43ontrol\x12\x0e\n\x06\x61gents\x18\x01 \x03(\t\"]\n\x07Message\x12\x41\n\x07\x63ontrol\x18\x01 \x01(\x0b\x32\x30.ostorlab.agent.message.proto.v3.control.Control\x12\x0f\n\x07message\x18\x02 \x01(\x0c')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.agent.message.proto.v3.control.control_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CONTROL._serialized_start=98
  _CONTROL._serialized_end=123
  _MESSAGE._serialized_start=125
  _MESSAGE._serialized_end=218
# @@protoc_insertion_point(module_scope)
